/* Elfbayne text adventure for the 16K RAM ZX80
by benkid77 (aka Digital Prawn)
Version 1.0 coded from 31/05/2019-17/06/2019 

Compile under Z88DK with:
zcc +zx80 -create-app -o elfbayne elfbayne.c 

Credits: 
Line input code based on Paul Taylor's (aka poglad)
1998 reverse-engineering of "Adventure A - Planet of Death"
to C source code.
Verb searching code loosely based on the BASIC listing of RUFAL
for the Grundy Newbrain by Andrew Pepper
*/

#include <stdio.h> 
#include <stdlib.h> 
#include <string.h> 

//Preprocessor directives
//#define DEBUG
//#define DEVMODE
#define MIN(a,b) (((a)<(b))?(a):(b))
#define NUM_COLUMNS 32
#define NUM_LOCATIONS 46
#define NUM_OBJECTS 55
#define NUM_ABBREVS 4
#define NUM_FLAGS 8

#define LOC_NOWHERE 0
#define LOC_PLAYER 0xff

#define OBJ_IMMOVABLE 0x01
#define OBJ_TREASURE 0x02
#define OBJ_ANIMATE 0x04

#define MAX_CARRIED_ITEMS 6

#define DIR_NORTH 0
#define DIR_EAST 1
#define DIR_SOUTH 2
#define DIR_WEST 3
#define DIR_UP 4
#define DIR_DOWN 5

#define FLAG_SCORE 0
#define FLAG_GAMEOVER 1
#define FLAG_GRASS 2
#define FLAG_BEE 3
#define FLAG_LADDGLUE 4

//typedefs
typedef unsigned char byte;
typedef unsigned short int word;
typedef unsigned char bool;
typedef struct
	{
	char *desc;
	byte dest[6];
	} location;

typedef struct
	{
	char *desc;
	byte initial_loc;
	byte initial_flags;
	} object;

enum {FALSE,TRUE};

//Function prototypes
void init_universe(void);
void init_vars(void);
void print_instructions(void);
void cls(void);
char my_getc(void);
void my_putc(char c);
void my_newline(void);
void my_newline2(void);
void my_print(char *s);
void my_print_message(byte m);
void my_getline(void);
bool my_parseline(void);
byte find_word(char *word_list,char *word);
void print_location_description(void);
void print_directions(void);
void print_items(byte l);
void perform_player_action(void);
void verb_move_player(byte d);
void verb_inventory(void);
void verb_take(void);
bool is_in_room(byte o);
bool is_carried(byte o);
bool is_in_scope(byte o);
void verb_drop(void);
void verb_examine(void);
void verb_look(void);
void verb_light(void);
void verb_melt(void);
void verb_dig(void);
void dig_up_object(byte l,byte o);
void verb_climb(void);
void verb_tie(void);
void verb_fire(void);
void verb_kill(void);
void verb_shoot(void);
void game_over(void);
void game_won(void);
void play_again_or_exit(void);
byte get_y_n(void);
void verb_move(void);
void verb_wave(void);
#ifdef DEVMODE
void verb_debug(void);
byte str2byte(char *s);
#endif
void verb_quit(void);
void verb_restart(void);
void verb_score(void);
void verb_fish(void);
void verb_dive(void);
void verb_blow(void);
void verb_tickle(void);
void verb_open(void);
void verb_unlock(void);
void verb_smash(void);
void verb_glue(void);
void verb_hammer(void);
void verb_give(void);
void final_score(void);
void print_notifications(void);
void verb_swim(void);

//Variables
//text input buffer
char input_buf[NUM_COLUMNS];
//location
byte lo;
//describe room?
bool dr;
//verb and noun of current command
char verb[5];
char noun[5];
//verb number and noun number
byte vn, nn;
//current object where applicable
byte cob;
//items carried
byte ic;
//flags
byte flag[NUM_FLAGS];
//number of turns
word turns;

//Static data
char *game_instructions=
	"welcome to ELFBAYNE\n"
	"for the 16k ram zx80\n\n"
	"in this game you must retrieve\n"
	"the twelve lost TREASURES of\n"
	"tivenboll and place them in the\n"
	"diamond room.\n"
	"bellrond will collect,\n"
	"ta very much.\n"
	"p.s. be careful out there elf.\n\n"
	"press a key to start\n";

//Abbreviations for room descriptions only
//first char is the key, 
//second char onwards is the expanded string
char *abbrev[NUM_ABBREVS]=
{
	"Yyou are in ",
	"Oyou are on ",
	"Ayou are ",
	"Dyou are in the dire maze"
};

//Directions are N, E, S, W, U, D
location loc[NUM_LOCATIONS+1]=
{
	{NULL,{0,0,0,0,0,0}}, 					//00
	{"Yyour cottage",{2,0,0,0,0,0}},	 		//01
	{"Yyour garden",{3,0,1,0,0,0}}, 			//02
	{"Ya large field",{0,8,2,4,0,0}}, 			//03
	{"Ya peaceful valley",{0,3,5,0,0,0}}, 			//04
	{"Yan ancient forest",{4,0,0,46,0,0}},			//05
	{"Ya magical wood",{5,0,7,0,0,0}},			//06
	{"Ya small clearing",{6,0,0,0,0,0}},			//07
	{"Obramble hill",{9,12,0,3,0,0}},			//08
	{"Oa mountain path",{0,0,8,10,13,0}},			//09
	{"Aoutside a cave",{0,9,0,0,0,0}},			//10
	{"Ya smoky cave",{0,10,0,0,0,0}},			//11
	{"Ahalfway up the mountain",{13,14,0,8,0,0}},		//12
	{"Otop of misty mountain",{0,0,12,0,0,9}},		//13
	{"Oa snowy pass",{0,0,0,12,0,0}},			//14
	{"Oa cliff edge",{0,0,14,0,0,0}},			//15
	{"Ya sloping gorge",{0,17,18,0,0,0}},			//16
	{"Oa rocky plateau",{0,0,19,0,0,16}},			//17
	{"Ya swamp",{16,0,0,0,0,0}},				//18
	{"Oa zigzag path",{17,0,20,0,0,0}},			//19
	{"Oa sandy beach",{19,0,21,0,25,0}},			//20
	{"Ythe sea",{20,0,23,0,0,22}},				//21
	{"Othe seabed",{0,0,0,0,21,0}},				//22
	{"Oa pebble beach",{21,0,0,0,0,0}},			//23
	{"Oa tiny islet",{23,0,0,0,0,0}},			//24
	{"Oa windswept plain",{0,0,0,26,0,20}},			//25
	{"Aoutside a dark citadel",{0,25,0,27,0,0}},		//26
	{"Ya grand hallway",{0,26,0,0,0,0}},			//27
	{"Ya dark corridor",{0,27,29,0,0,0}},			//28
	{"D",{28,29,29,29,29,30}},				//29
	{"D",{29,31,32,29,29,29}},				//30
	{"D",{29,29,29,33,29,29}},				//31
	{"D",{29,29,35,29,29,29}},				//32
	{"D",{29,29,29,29,34,29}},				//33
	{"D",{29,29,39,29,29,29}},				//34
	{"D",{29,36,29,29,29,29}},				//35
	{"D",{29,29,37,29,29,29}},				//36
	{"Ythe emerald room",{36,0,0,38,0,0}},			//37
	{"Ythe diamond room",{0,37,0,0,0,0}},			//38
	{"Ya curved tunnel",{34,0,0,40,0,0}},			//39
	{"Oa flowery path",{0,39,0,41,0,0}},			//40
	{"Anear a sandstone wall",{0,40,0,0,0,0}},		//41
	{"Oyour cottage roof",{0,0,0,0,0,2}},			//42
	{"Ysmall hollow",{0,0,0,0,24,0}},			//43
	{"Ainside the belly of a\nwhale",{0,0,0,0,21,0}},	//44
	{"Aup an oak tree",{0,0,0,0,0,15}},			//45
	{"Ya smithy",{0,5,0,0,0,0}}				//46
};


//Define in-game objects and initial locations and states
object ob[NUM_OBJECTS+1]=
{
	{NULL,LOC_NOWHERE,0}, 							//00
	{"a wooden bed",1,OBJ_IMMOVABLE}, 					//01
	{"a note",LOC_NOWHERE,0}, 						//02
	{"a bumble bee",2,OBJ_ANIMATE},						//03
	{"a yellow flower",40,0},						//04
	{"a brown bear",10,OBJ_IMMOVABLE|OBJ_ANIMATE},				//05
	{"a huge whale",21,OBJ_IMMOVABLE|OBJ_ANIMATE},				//06
	{"a rocky plateau to the east",16,OBJ_IMMOVABLE},			//07
	{"an orc standing on the plateau",16,OBJ_IMMOVABLE|OBJ_ANIMATE},	//08
	{"a sleeping giant",23,OBJ_IMMOVABLE|OBJ_ANIMATE},			//09
	{"some grass",24,OBJ_IMMOVABLE},					//10
	{"a closed grate",LOC_NOWHERE,OBJ_IMMOVABLE},				//11
	{"an open grate",LOC_NOWHERE,OBJ_IMMOVABLE},				//12
	{"a mithril helmet",27,0},						//13
	{"a bald puzzled-looking dwarf",7,OBJ_IMMOVABLE|OBJ_ANIMATE},		//14
	{"an elderly bearded wizard",26,OBJ_IMMOVABLE|OBJ_ANIMATE},		//15
	{"a feather",39,0},							//16
	{"a torch (unlit)",1,0},						//17
	{"A\x40""PLATINUM\x40""BROOCH",LOC_NOWHERE,OBJ_TREASURE},		//18
	{"a bow",4,0},								//19
	{"some shrubs",5,OBJ_IMMOVABLE},					//20
	{"a magic wand",6,0},							//21
	{"A\x40""SNUFFBOX",LOC_NOWHERE,OBJ_TREASURE},				//22
	{"a hammer",8,0},							//23
	{"an iron key",9,0},							//24
	{"A\x40""DIAMOND",11,OBJ_TREASURE},					//25
	{"A\x40""CROWN",13,OBJ_TREASURE},					//26
	{"a wall of ice",14,OBJ_IMMOVABLE},					//27
	{"a solitary oak tree",15,OBJ_IMMOVABLE},				//28
	{"A\x40""GOLD\x40""RING",18,OBJ_TREASURE},				//29
	{"AN\x40""EMERALD",LOC_NOWHERE,OBJ_TREASURE},				//30
	{"A\x40""NECKLACE",22,OBJ_TREASURE},					//31
	{"A\x40""RUBY",LOC_NOWHERE,OBJ_TREASURE},				//32
	{"a fishing rod",43,0},							//33
	{"a locked door",27,OBJ_IMMOVABLE},					//34
	{"a pot of glue",28,0},							//35
	{"A\x40""PAINTING",37,OBJ_TREASURE},					//36
	{"a glass frame",37,OBJ_IMMOVABLE},					//37
	{"a broken ladder",41,OBJ_IMMOVABLE},					//38
	{"A\x40""SILVER\x40""BAR",44,OBJ_TREASURE},				//39
	{"a fire",46,OBJ_IMMOVABLE},						//40
	{"A\x40""BRACELET",46,OBJ_TREASURE},					//41
	{"A\x40""HARP",27,OBJ_TREASURE},					//42
	{"a torch (lit)",LOC_NOWHERE,0},					//43
	{"a shovel",2,0},							//44
	{"a rope",12,0},							//45
	{"a gorge below the cliff",15,OBJ_IMMOVABLE},				//46
	{"a rope tied to the tree",LOC_NOWHERE,OBJ_IMMOVABLE},			//47
	{"an arrow",45,0},							//48
	{"a stone dead dwarf",LOC_NOWHERE,OBJ_IMMOVABLE},			//49
	{"a hunting horn",25,0},						//50
	{"an open door",LOC_NOWHERE,0},						//51
	{"a smashed glass frame",LOC_NOWHERE,OBJ_IMMOVABLE},			//52
	{"a shimmering forcefield",38,OBJ_IMMOVABLE},				//53
	{"a bumble bee (on flower)",LOC_NOWHERE,0},				//54
	{"a fixed ladder",LOC_NOWHERE,OBJ_IMMOVABLE}				//55
};

//Current location of objects - changes in-game
byte oblo[NUM_OBJECTS+1];


//The order of the first 12 verbs (directional verbs) are in fixed positions
//n,nort,e,east,s,sout,w,west,u,up,d,down
//They are also used in the "Obvious exits" printing routine and to index
//the destination array in each room
char *verb_list=
	"n   norte   easts   soutw   west"	//1-8
	"u   up  d   downi   inv invecarr"	//9-16
	"get takedropput examsearl   look"	//17-24
        "desclighmeltdig climtie firekill"	//25-32
	"shoomovewavedbugquitrestscorfish"	//33-40
	"diveblowtickopenclosunlosmasbrea"	//41-48
	"gluehammfix givec   x   exa help"	//49-56
	"infosay xyzzz   waitswimpushpull"	//57-64
	"thro";

char *noun_list=
	"woodbed notebumbbee yellflowbrow"	//1-8
	"bearhugewhalorc sleegiangrasgrat"	//9-16
	"mithhelmbalddwareldewizaice wall"	//17-24
	"feattorcplatbroobow shrumagiwand"	//25-32
	"snufhammironkey diamcrowoak tree"	//33-40
	"goldringemerneckrubyfishrod lock"	//41-48
	"doorpot gluepainglasframladdsilv"	//49-56
	"bar firebracharpshovropearro.mov"	//57-64
	".gethunthorn.nic";

//map nouns to objects where applicable
//use 0 where a noun does not map to an object
byte noun2ob[]=
	{0,
	1,1,2,3,3,4,4,5,		//1-8
	5,6,6,8,9,9,10,11,		//9-16
	13,13,14,14,15,15,27,27,	//17-24
	16,17,18,18,19,20,21,21,	//25-32
	22,23,24,24,25,26,28,28,	//33-40
	29,29,30,31,32,33,33,34,	//41-48
	34,35,35,36,37,37,38,39,	//49-56
	39,40,41,42,44,45,48,0,		//57-64
	0,50,50,0
	};

char *message[]={
	"ok",					//00
	"sorry, i do not understand",		//01
	"you cannot go that way",		//02
	"you cannot do that",			//03
	"you see nothing special",		//04
	"you cannot carry any more",		//05
	"the bee would rather be left\n"	//06
	"alone at present",
	"it is already lit",			//07
	"you cannot do that yet",		//08
	"the wall of ice is in the way",	//09
	"the ice wall melts away",		//10
	"you find nothing",			//11
	"you found something",			//12
	"you cannot do that here",		//13
	"try climbing",				//14
	"climb what?",				//15
	"you should tie it first",		//16
	"the rope comes loose and falls\n"	//17
	"into the gorge, but at least\n"
	"you have landed safely",
	"you fire an arrow, it lands\n"		//18
	"nearby",
	"how?",					//19
	"you need something to shoot with",	//20
	"you decide not to waste an arrow",	//21
	"at such close range, the bear\n"	//22
	"swipes the bow out of your hand\n"
	"before killing you",
	"\n*** you are dead ***",		//23
	"\n*** you have won ***\n",		//24
	"would you like to play again?\n"	//25
	"(y/n)",
	"you head towards the orc, alas\n"	//26
	"he kills you before you can\n"
	"reach the plateau",
	"a precision shot sends the orc\n"	//27
	"tumbling off the plateau, his\n"
	"body landing somewhere out of\n"
	"sight",
	"you move the shrubs, revealing\n"	//28
	"a hidden path to the south",
	"you have just murdered an\n"		//29
	"innocent dwarf",
	"you are right up a creek now...",	//30
	"the wizard prevents mere novices"	//31
	"of magick from progressing past\n"
	"this point",
	"nothing happens",			//32
	"the wizard sternly tells you\n"	//33
	"that is no way to wave a wand\n"
	"he takes it from you and shows\n"
	"you how it is done ... and with\n"
	"a puff of smoke both he and the\n"
	"wand are gone",
	"are you sure you want to quit?\n"	//34
	"(y/n)",
	"are you sure you want to\n"		//35
	"restart? (y/n)",
	"as you disturb the gold ring\n"	//36
	"it sinks into the swamp, lost\n"
	"forever",
	"you try to fish the ring out but",	//37
	"you carefully fish the ring out\n"	//38
	"of the swamp and pocket it",
	"the whale prevents you from\n"		//39
	"diving down and swallows you",
	"you blow the horn loudly",		//40
	"the whale speedily swims away",	//41
	"you would rather not lose\n"		//42
	"anything here",
	"you tickle the giant with the\n"	//43
	"feather - he wakes up and stomps"
	"off, revealing a hidden object",
	"the sleeping giant blocks the\n"	//44
	"way south",
	"you find a grate hidden in the\n"	//45
	"grass",
	"you unlock the door with the key"	//46
	"and open it",
	"it is stuck in the frame",		//47
	"you smash the frame to pieces",	//48
	"TREASURE safely stored",		//49
	"the forcefield prevents it",		//50
	"the bee lands on the flower",		//51
	"the bumble bee flies off",		//52
	"and buzzes around the bear\n"		//53
	"causing him to run away",
	"it is already glued",			//54
	"you glue the ladder, but it is\n"	//55
	"still a bit loose",
	"the ladder is now fixed",		//56
	"it needs sticking together first",	//57
	"the dwarf thanks you before\n"		//58
	"dropping an item and leaving",
	"congratulations - you have\n"		//59
	"retrieved all of the twelve\n"
	"lost treasures\n\n"
	"the elders of tivenboll will\n"
	"remember your outstanding\n"
	"service\n\n"
	"bellrond personally rewards you\n"
	"with the elven medal of honour",
	"the torch has been extinguished",	//60
	"\npress a key",			//61
	"collect the 12 TREASURES and\n"	//62
	"drop them in the diamond room",
	"elfbayne v1.0 rel 17/06/2019\n"	//63
	"by benkid77 - this game is\n"
	"public domain",
	"perhaps they can be moved out of"	//64
	"the way",
	"you fish for a while...",		//65
	"you wait..."				//66
	};

int main() 
{ 

init_universe();

for(;;) //restart game loop
	{
	init_vars();
	print_instructions();
	cls();

	while(!flag[FLAG_GAMEOVER]) //main game loop
		{
		if(dr) 
			{
			print_location_description();
			print_notifications();
			print_directions();
			my_print("you can see:\n");
			print_items(lo);
			}
		dr=FALSE;
		do
			{
			my_getline();
			} while(!my_parseline());

		#ifdef DEBUG
			printf("DEBUG: verb=%s noun=%s vn=%d nn=%d\n",verb,noun,vn,nn);
		#endif
			if(vn)
				perform_player_action();
			else
				my_print_message(1);

		turns++;
		} //end main game loop
	//game over
	//Player has died if gameover flag==1
	//Game won if gameover flag==2
	//flag==3 if player has confirmed to quit
	//flag==4 if player has confirmed to restart
	switch(flag[FLAG_GAMEOVER])
		{
		case 1:
			game_over();
			final_score();
			play_again_or_exit();
			cls();
			break;
		case 2:
			game_won();
			final_score();
			play_again_or_exit();
			cls();
			break;
		case 3:
			final_score();
			cls();
			exit(0);

		case 4:
			final_score();
		default:
			break;
		}
	} //end restart game loop
return 0;
}

void init_universe(void)
{
//NUL terminate verb and noun buffers
verb[4]=noun[4]=0;
}

void init_vars(void)
{
byte i;

//reset turn counter
turns=0;

//set starting location
lo=1;

//set describe room flag
dr=TRUE;

//set number of items carried
ic=0;

//reset locations of all objects
for(i=0;i<(NUM_OBJECTS+1);i++)
	{
	oblo[i]=ob[i].initial_loc;
	}

//reset flags
for(i=0;i<NUM_FLAGS;i++)
	flag[i]=0;

//reset special items
noun2ob[26]=17;	//set "torch" to refer to unlit torch
noun2ob[55]=38; //set "ladder" to refer to broken ladder
noun2ob[62]=45; //set "rope" to refer to untied rope

//reset obstacles
loc[14].dest[DIR_NORTH]=0; //set ice wall exists
loc[5].dest[DIR_SOUTH]=0;  //set forest path hidden
loc[23].dest[DIR_SOUTH]=0; //set sleeping giant blocking exit
loc[24].dest[DIR_DOWN]=0; //set grate closed
loc[27].dest[DIR_WEST]=0; //set door locked
loc[10].dest[DIR_WEST]=0; //set bear blocking cave entrance
}

void print_instructions(void)
{
int i;
cls();
my_print(game_instructions);
my_getc();
}

void cls(void)
{
my_putc(12);
}

char my_getc(void)
{
return fgetc_cons();
} 

void my_putc(char c)
{
putchar(c);
}

void my_newline(void)
{
my_putc('\n');
}

void my_newline2(void)
{
my_newline();my_newline();
}

void my_print(char *s)
{
printf(s);
}

void my_print_message(byte m)
{
my_print(message[m]);
my_newline();
}

void my_getline(void)
{
byte i=0;
char c=0;
my_putc('>');

while(c!='\n')
	{
	input_buf[i]=0;
	c=my_getc();

	if((c=='\b')||(c==12))  //delete or backspace
		{
		if(i>0)  //if input_buf not empty then delete last char 
			{
			i--;
			my_putc(8);
			my_putc(' ');
			my_putc(8);
			}
		}
	else if(c!='\n')  //any ordinary character
		{
		if(i==(NUM_COLUMNS-1))  //prevent going beyond end of line
			{
			my_putc(' ');
			my_putc(8);
			}
		else
			{
			my_putc(c);
			input_buf[i++]=c;
			}
		}
	}
	my_putc('\n');
}

bool my_parseline(void)
{
char *token;

//initially overwrite verb and noun buffers with spaces
memset(verb,32,4);
memset(noun,32,4);

//reset current object, verb and noun numbers
cob=vn=nn=0;

if(!(token=strtok(input_buf," ")))
	return FALSE;

strncpy(verb,token,MIN(strlen(token),4));
vn=find_word(verb_list,verb);

if(token=strtok(NULL," "))
	{
	strncpy(noun,token,MIN(strlen(token),4));
	nn=find_word(noun_list,noun);
	cob=noun2ob[nn];

	//if object is not in scope then it can
	//be immediately disregarded here
	if(!is_in_scope(cob)) cob=0;
	}

return TRUE;
}

byte find_word(char *word_list,char *word)
{
word i;
for (i=0;i<strlen(word_list);i+=4)
	{
	if(!strncmp(&word_list[i],word,4)) return (byte)((i/4)+1);
	}
return 0;
}

void print_location_description()
{
char *c=loc[lo].desc;
char *a;
byte i;

my_newline();

while(*c)
	{
	//Expand any abbreviations (capital letters)
	if((*c>='A')&&(*c<='Z'))
		{
		for(i=0;i<NUM_ABBREVS;i++)
			{
			a=abbrev[i];
			if(*a==*c)
				{
				my_print(a+1);
				break;
				}
			}
		}
	else
		my_putc(*c);
	
	c++;
	}

my_newline2();
}

void print_directions(void)
{
byte i;
bool f=FALSE;

my_print("obvious exits: ");

for(i=0;i<6;i++)
	{
	if(loc[lo].dest[i])
		{
		my_putc(verb_list[i<<3]);
		my_putc(' ');
		f=TRUE;
		}
	}

if(!f) my_print("(none)");

my_newline2();
}

void print_items(byte l)
{
byte i;
byte f=FALSE;

for(i=1;i<=NUM_OBJECTS;i++)
	if(oblo[i]==l) 
		{
		my_print(ob[i].desc);
		my_newline();
		f=TRUE;
		}

if(!f)
	{
	my_print("(nothing)");
	my_newline();
	}

my_newline();
}

void perform_player_action(void)
{
if(vn<13)
	{
	//this verb covers the six main directions n,e,s,w,u,d
	//convert the verb number 1-12
	//into a direction number 0-5
	verb_move_player((vn-1)>>1);
	}
else
	{
	switch(vn)
		{
		case 13:
		case 14:
		case 15:
		case 16:
		case 53:
			verb_inventory();
			break;
		case 17:
		case 18:
			verb_take();
			break;
		case 19:
		case 20:
		case 65:
			verb_drop();
			break;
		case 21:
		case 22:
		case 54:
		case 55:
			verb_examine();
			break;
		case 23:
		case 24:
		case 25:
			verb_look();
			break;
		case 26:
			verb_light();
			break;
		case 27:
			verb_melt();
			break;
		case 28:
			verb_dig();
			break;
		case 29:
			verb_climb();
			break;
		case 30:
			verb_tie();
			break;
		case 31:
			verb_fire();
			break;
		case 32:
			verb_kill();
			break;
		case 33:
			verb_shoot();
			break;
		case 34:
			verb_move();
			break;
		case 35:
			verb_wave();
			break;
		case 36:
#ifdef DEVMODE
			verb_debug();
#else
			my_print_message(1);
#endif
			break;
		case 37:
			verb_quit();
			break;
		case 38:
			verb_restart();
			break;
		case 39:
			verb_score();
			break;
		case 40:
			verb_fish();
			break;
		case 41:
			verb_dive();
			break;
		case 42:
			verb_blow();
			break;
		case 43:
			verb_tickle();
			break;
		case 44:
			verb_open();
			break;
		case 46:
			verb_unlock();
			break;
		case 47:
		case 48:
			verb_smash();
			break;
		case 49:
			verb_glue();
			break;
		case 50:
			verb_hammer();
			break;
		case 51:	//fix
			my_print_message(19);	//how?
			break;
		case 52:
			verb_give();
			break;
		case 56:	//help
			my_print_message(62);
			break;
		case 57:	//info
			my_print_message(63);
			break;
		case 58: 	//say
			my_print_message(0);
			break;
		case 59:	//xyzzy
			my_print_message(32);
			break;
		case 60:	// z, wait
		case 61:
			my_print_message(66);
			break;
		case 62:
			verb_swim();
			break;
		default:
		case 45:	//close
		case 63:	//push
		case 64:	//pull
			my_print_message(3);
			break;
		}
	}
}

void verb_move_player(byte d)
{
byte nl;

//Special locations and obstacles
switch(lo)
	{
	//snowy pass
	case 14:
		if((d==DIR_NORTH)&&(loc[14].dest[DIR_NORTH]==0))
			{
			my_print_message(9);
			return;
			}
		break;

	//on a cliff edge
	case 15:
		if((d==DIR_UP)||(d==DIR_DOWN))
			{
			my_print_message(14);
			return;
			}
		break;

	//in a sloping gorge
	case 16:
		if((d==DIR_EAST)&&(oblo[8]==16))
			{
			flag[FLAG_GAMEOVER]=1;
			my_print_message(26);
			return;
			}
		break;

	//in the sea
	case 21:
		if((d==DIR_DOWN)&&(oblo[6]==21))
			{
			lo=44;
			dr=1;
			my_print_message(39);
			return;
			}
		break;

	//giant on pebble beach
	case 23:
		if((d==DIR_SOUTH)&&(oblo[9]==23))
			{
			my_print_message(44);
			return;
			}
		break;

	//outside a dark citadel
	case 26:
		if((d==DIR_WEST)&&(oblo[15]==26))
			{
			my_print_message(31);
			return;
			}
		break;

	default:
		break;
	}

if(nl=loc[lo].dest[d])
	{
	lo=nl;
	dr=TRUE;
	my_print_message(0);
	}
else
	my_print_message(2);
}

void verb_inventory(void)
{
my_print("you are carrying:\n");
print_items(LOC_PLAYER);
}

void verb_take(void)
{
if(is_in_room(cob))
	{
	//Diamond room treasures
	if((lo==38)&&(ob[cob].initial_flags&OBJ_TREASURE))
		{
		my_print_message(50);
		return;
		}

	//deal with special items
	switch(cob)
		{
		case 3:		//Bumble bee
			if(is_carried(4)&&(!flag[FLAG_BEE]))
				{
				flag[FLAG_BEE]=TRUE;
				oblo[3]=LOC_NOWHERE;
				oblo[54]=LOC_PLAYER;
				my_print_message(51);
				return;
				}
			my_print_message(6); //the bee would rather be left alone
			return;
		case 29:	//Gold ring
			if(lo==18)	//in swamp?
				{
				oblo[29]=LOC_NOWHERE;
				my_print_message(36);
				my_print_message(30);
				return;
				}
			break;
		case 36:	//Painting
			if(oblo[37]==37)	//is (unbroken) frame in the emerald room?
				{
				my_print_message(47);
				return;
				}
			break;
		default:
			break;
		}

	if(ob[cob].initial_flags&OBJ_IMMOVABLE)
		{
		my_print_message(3);
		}
	else
		{
		if(ic<MAX_CARRIED_ITEMS)
			{
			oblo[cob]=LOC_PLAYER;
			ic++;
			my_print_message(0);
			}
		else
			{
			my_print_message(5);
			}
		}
	}
else
	{
	my_print_message(3);
	}
}

bool is_in_room(byte o)
{
return (oblo[o]==lo);
}

bool is_carried(byte o)
{
return (oblo[o]==LOC_PLAYER);
}

//check if an item is in scope. i.e. is it in the current room
//or carried by the player
bool is_in_scope(byte o)
{
return is_in_room(o)||is_carried(o);
}

void verb_drop(void)
{
//exceptions
if((lo==44)||(lo==21)||(lo==22)) //in whale belly, sea or seabed
	{
	my_print_message(42);
	return;
	}

if(is_carried(cob))
	{
	oblo[cob]=lo;
	ic--;
	if((lo==38)&&(ob[cob].initial_flags&OBJ_TREASURE)) //Dropping treasure in the diamond room?
			{
			flag[FLAG_SCORE]++;
			my_print_message(49);
			if(flag[FLAG_SCORE]==12)	//won game when all 12 items dropped
				flag[FLAG_GAMEOVER]=2;
			return;
			}
	if((cob==4)&&flag[FLAG_BEE]) //Dropping flower with bee stuck on it?
			{
			oblo[54]=LOC_NOWHERE;
			my_print_message(52);
			if((lo==10)&&(oblo[5]==10)) //outside cave with bear here?
				{
				oblo[5]=LOC_NOWHERE;
				loc[10].dest[DIR_WEST]=11;
				my_print_message(53);
				}
			else
				my_print_message(30);
			return;
			}
	my_print_message(0);
	}
else
	my_print_message(3);
}

void verb_examine(void)
{
if((cob==10)&&(!flag[FLAG_GRASS]))
	{
	oblo[11]=24;
	flag[FLAG_GRASS]=TRUE;
	my_print_message(45);
	return;
	}

if((cob==20)&&(!loc[5].dest[DIR_SOUTH]))
	{
	my_print_message(64);
	return;
	}

my_print_message(4);
}

void verb_look(void)
{
dr=TRUE;
my_print_message(0);
}

void verb_light(void)
{
if(is_carried(cob))
	{
	switch(cob)
		{
		//unlit torch
		case 17:
			if(lo==46)
				{
				oblo[17]=LOC_NOWHERE;
				oblo[43]=LOC_PLAYER;
				noun2ob[26]=43;
				my_print_message(0);
				}
			else
				my_print_message(8);
			break;
		//lit torch
		case 43:
			my_print_message(7);
			break;
		default:
			my_print_message(3);
			break;
		}
	}
else
	my_print_message(3);
}

void verb_melt(void)
{
//wall of ice
if(cob==27)
	{
	if(is_carried(43))
		{
		oblo[27]=LOC_NOWHERE;
		loc[14].dest[DIR_NORTH]=15;
		my_print_message(10);
		}
	else
		my_print_message(8);
	}
else
	my_print_message(3);
}

void verb_dig(void)
{
//return if player not carrying shovel
if(!is_carried(44))
	{
	my_print_message(8);
	return;	
	}

switch(lo)
	{
	//dig up brooch in large field
	case 3:
		dig_up_object(3,18);
		break;

	//dig up emerald on sandy beach
	case 20:
		dig_up_object(20,30);
		break;

	//all other locations		
	default:
		my_print_message(11);
		break;
	}
}

void dig_up_object(byte l,byte o)
{
if(oblo[o]==LOC_NOWHERE)
	{
	oblo[o]=l;
	my_print_message(12);
	}
else
	my_print_message(11);
}

void verb_climb(void)
{
if(!cob)
	{
	my_print_message(15);
	return;
	}

switch(lo)
	{
	//on a cliff top
	case 15:
		switch(cob)
			{
			case 28:	//oak tree
				lo=45;
				dr=1;
				my_print_message(0);
				break;
			case 45:	//untied rope
				my_print_message(16);
				break;
			case 47:	//rope tied to tree
				lo=16;
				dr=1;
				oblo[47]=LOC_NOWHERE;	//rope comes loose
				oblo[45]=16;		//and falls into gorge
				noun2ob[62]=45;		//"rope" refers to loose rope again
				my_print_message(0);
				my_print_message(17);
				break;
			default:
				my_print_message(3);
				break;	
			}
		break;
	case 41:	//near a sandstone wall
		switch(cob)
			{
			case 38: //broken ladder
				my_print_message(8);
				break;
			case 55: //fixed ladder
				lo=42;
				dr=1;
				my_print_message(0);
				break;
			default:
				my_print_message(3);
				break;
			}		
		break;
	default:
		my_print_message(3);
		break;
	}
}

void verb_tie(void)
{
if(cob!=45)	//return if not rope
	{
	my_print_message(3);
	return;
	}

if(lo!=15)    //return if not on cliff egde
	{
	my_print_message(13);
	return;
	}

//decrement number of carried items if rope
//is in player inventory before tying
if(is_carried(45))
	ic--;

oblo[45]=LOC_NOWHERE;
oblo[47]=15;
noun2ob[62]=47; //noun "rope" now refers to tied rope
my_print_message(0);
}

void verb_fire(void)
{
if((cob==19)||(cob==48)) //if "fire bow" or "fire arrow"
	{
	if(is_carried(19)&&is_carried(48))	//if player has bow and arrow
		{
		oblo[48]=lo;
		ic--;
		my_print_message(0);
		my_print_message(18);
		return;
		}
	}
my_print_message(3);
}

void verb_kill(void)
{
if(ob[cob].initial_flags&OBJ_ANIMATE)
	my_print_message(19);
else
	my_print_message(3);
}

void verb_shoot(void)
{
if(is_carried(19)&&is_carried(48))	//if player has bow and arrow
	{
	if(is_carried(cob))		//trying to shoot something being carried
		{
		if((cob==19)||(cob==48))	//"shoot bow" or "shoot arrow"
			{
			oblo[48]=lo;
			ic--;
			my_print_message(0);
			my_print_message(18);
			}
		else
			my_print_message(3);	//otherwise can't shoot something being carried
		}
	else		//shooting something in current location
		{
		switch(cob)
			{
			case 5:	//bear
				flag[FLAG_GAMEOVER]=1;
				my_print_message(22);
				break;

			case 8: //orc
				oblo[8]=LOC_NOWHERE;
				oblo[48]=LOC_NOWHERE;
				ic--;
				my_print_message(27);
				break;
			case 14: //dwarf
				oblo[14]=LOC_NOWHERE;
				oblo[48]=LOC_NOWHERE;
				ic--;
				oblo[49]=lo;
				my_print_message(0);
				my_print_message(29);
				my_print_message(30);
				break;
			default:
				my_print_message(21);
				break;
			}
		}
	}
else		//player does not have bow and/or arrow
	my_print_message(20);
}

void game_over(void)
{
my_print_message(23);
}

void game_won(void)
{
my_print_message(24);
my_print_message(59);
}

void play_again_or_exit(void)
{
my_print_message(25);
if(get_y_n()=='y')
	return;
else
	exit(0);
}

byte get_y_n(void)
{
char c;

do
	{
	c=my_getc();
	}	while((c!='y')&&(c!='n'));

return c;
}

void verb_move()
{
if((cob==20)&&(!loc[5].dest[DIR_SOUTH]))
	{
	my_print_message(28);
	loc[5].dest[DIR_SOUTH]=6;
	}
else
	my_print_message(3);
}

void verb_wave(void)
{
if(cob==21)	//magic wand
	{
	if((lo==26)&&(oblo[15]==26))

		{
		oblo[15]=LOC_NOWHERE;
		if(is_carried(21))		
			ic--;
		oblo[21]=LOC_NOWHERE;		
		my_print_message(0);
		my_print_message(33);
		}
	else
		{
		my_print_message(0);
		my_print_message(32);	
		}	
	}
else
	my_print_message(3);
}

#ifdef DEVMODE
void verb_debug(void)
{
byte v;
char *my_arg=strtok(NULL," ");

v=str2byte(my_arg);

//printf("noun=%s nn=%d my_arg=%s v=%d\n", noun, nn, my_arg, v); 

switch(nn)
	{
	case 64:	//.mov
		lo=v;
		dr=1;
		my_print_message(0);
		return;
	case 65:	//.get
		oblo[v]=LOC_PLAYER;
		ic++;
		my_print_message(0);
		return;
	case 68:	//.nic
		printf("%d\n",ic);
		return;
	default:
		return;
	}
}

byte str2byte(char *s)
{
byte i,l,m,v;

if(!s) return 0;

if(!(l=strlen(s))) return 0;

for(i=l,v=0,m=1;i;i--,m*=10)
	{
	v+=((s[i-1]-'0')*m);
	}

return v;
}
#endif

void verb_quit(void)
{
my_print_message(34);
if(get_y_n()=='y')
	flag[FLAG_GAMEOVER]=3;
else
	my_print_message(0);
}

void verb_restart(void)
{
my_print_message(35);
if(get_y_n()=='y')
	flag[FLAG_GAMEOVER]=4;
else
	my_print_message(0);
}

void verb_score(void)
{
printf("you have stored %d out of 12\n",flag[FLAG_SCORE]);
printf("treasures in %d turns\n",turns);
}

void verb_fish(void)
{
if((cob==29)&&(oblo[29]==18))	//gold ring in the swamp?
	{
	if(is_carried(33)) //player has fishing rod?
		{
		if(ic<MAX_CARRIED_ITEMS) //player has space in inventory?
			{
			oblo[29]=LOC_PLAYER;
			ic++;
			my_print_message(38);
			}
		else
			{
			my_print_message(37);
			my_print_message(32);
			}
		}
	else
		my_print_message(8);
	}
else if(is_carried(33)&&(lo>=20)&&(lo<=24)) //in and around the "sea" area
	{
	my_print_message(65);
	my_print_message(3);
	}
else
	my_print_message(3);
}

void verb_dive()
{
if(lo==21)
	verb_move_player(DIR_DOWN);
else
	my_print_message(13);
}

void verb_blow()
{
if((cob==50)&&is_carried(50)) //hunting horn, and player his carrying it?
	{
	my_print_message(40);
	if((lo==21)&&(oblo[6]==21)) //in the sea and whale here?
		{
		oblo[6]=LOC_NOWHERE;
		my_print_message(41);
		if(oblo[39]==44) //silver bar still in whale belly?
			my_print_message(30);
		}
	}
else
	my_print_message(3);
}

void verb_tickle(void)
{
if(is_carried(16)) //player has feather?
	{
	if(cob==9)
		{
		oblo[9]=LOC_NOWHERE;
		oblo[32]=23;	//reveal ruby
		loc[23].dest[DIR_SOUTH]=24;
		my_print_message(43);
		}
	else
		my_print_message(13);
	}
else
	my_print_message(8);
}

void verb_open(void)
{
if(cob==11)
	{
	oblo[11]=LOC_NOWHERE;
	oblo[12]=24;
	loc[24].dest[DIR_DOWN]=43;
	my_print_message(0);
	return;
	}

my_print_message(3);
}

void verb_unlock(void)
{
if(cob==34)
	{
	if(is_carried(24))
		{
		oblo[34]=LOC_NOWHERE;
		oblo[51]=27;
		loc[27].dest[DIR_WEST]=28;
		my_print_message(46);
		}
	else
		{
		my_print_message(8);
		}
	}
else
	my_print_message(3);
}

void verb_smash(void)
{
if(cob==37)
	{
	if(is_carried(23))
		{
		oblo[37]=LOC_NOWHERE;
		oblo[52]=37;
		my_print_message(48);
		}
	else
		my_print_message(8);
	}
else
	my_print_message(3);
}

void verb_glue(void)
{
if(cob==38) //"glue ladder"
	{
	if(is_carried(35)) //carrying glue?
		{
		if(flag[FLAG_LADDGLUE]) //already glued?
			my_print_message(54);
		else
			{
			flag[FLAG_LADDGLUE]=TRUE;
			my_print_message(55);
			}
		}	
	else
		my_print_message(8);
	}
else
	my_print_message(3);
}

void verb_hammer(void)
{
if(cob==38) //"hammer ladder"
	{
	if(is_carried(23)) //carrying hammer?
		{
		if(flag[FLAG_LADDGLUE]) //ladder glued?
			{
			oblo[38]=LOC_NOWHERE;
			oblo[55]=41;
			noun2ob[55]=55;
			my_print_message(56);
			}
		else
			my_print_message(57);
		}
	else
		my_print_message(8);
	}
else
	my_print_message(3);
}

void verb_give(void)
{
if((cob==13)&&is_carried(13)&&(lo==7)) //"give helmet" in small clearing
	{
	ic--;
	oblo[13]=LOC_NOWHERE;
	oblo[14]=LOC_NOWHERE;
	oblo[22]=7; //snuffbox
	my_print_message(58);
	}
else
	my_print_message(3);
}

void final_score(void)
{
my_newline();
verb_score();
my_print_message(61);
my_getc();
my_newline();
}

void print_notifications(void)
{
if((lo==21)&&is_carried(43)) //if entered sea and carrying lit torch
	{
	//extinguish torch
	oblo[43]=LOC_NOWHERE;
	oblo[17]=LOC_PLAYER;
	noun2ob[26]=17;
	my_print_message(60);
	my_newline();
	}
}

void verb_swim(void)
{
if((lo==21)||(lo==22)) //in the sea or seabed
	my_print_message(0);
else
	my_print_message(3);
	
}

